@@ -51,8 +51,7 @@ def raise_version(version):
 def check_tests():
     """runs the tests over the package"""
     try:
-        # XXX get the env python
-        base.check_call('python setup.py ztest', shell=True)
+        base.check_call('%s setup.py test' % sys.executable, shell=True)
         return True
     except base.CalledProcessError:
         return False
@@ -160,42 +159,15 @@ def _run_setup(*args):
     del sys.modules['setup']
     sys.argv = old_args
 
-def pypi_upload():
+def pypi_upload(interactive=True):
     """upload into pypi"""
     _checkout_tag()
-    # let's upload on selected servers
-    if os.environ.has_key('HOME'): 
-        rc = os.path.join(os.environ['HOME'], '.pypirc')
-        if not os.path.exists(rc):
-            return 
-        config = ConfigParser()
-        config.read(rc)
-        sections = config.sections()
-
-        if 'distutils' in sections:
-            index_servers = config.get('distutils', 'index-servers')
-            _servers = [server.strip() for server in 
-                        index_servers.split('\n')
-                        if server.strip() != '']
-            for server in _servers:
-                if config.has_option(server, 'repository'):
-                    repository = config.get(server, 'repository')
-                else:
-                    repository = 'http://pypi.python.org/pypi'
-                _pypi_upload(repository)
-
-        elif 'server-login' in sections:
-            if config.has_option(server, 'repository'):
-                repository = config.get(server, 'repository')
-            else:
-                repository = 'http://pypi.python.org/pypi' 
-            _pypi_upload(repository)     
-        
-def _pypi_upload(repository):
-    res = raw_input('Do you want to upload to %s ? (Y/n):' % repository)
-
-    res = res.lower().strip()
+    if not interactive:
+        res = 'y'
+    else:
+        res = raw_input('Do you want to upload to the private repo ? (Y/n):')
+        res = res.lower().strip()
     if res == 'y':
         _run_setup('mregister', 'sdist', 'bdist_egg', 'mupload', '-r',
-                   repository)
+                   'ingeniweb-private')
 
