@@ -18,13 +18,16 @@ import ConfigParser
 import os
 import re
 import sys
-
+from tempfile import mkdtemp
 import base
 
 join = os.path.join
 
 extract_path = re.compile(r'^(htt(?:p|ps)://)(.*?)/?$')
 
+def _log(msg):
+    print msg
+
 def make_release(version=None):
     """this is called when a release is made over a buildout"""
 
@@ -88,47 +91,73 @@ def check_python(valid_version=(2, 4)):
         return '.'.join([str(v) for v in version])
 
 
+def diff_releases(old=None, new=None, result=None):
+    """takes two tarballs, and generates a diff one"""
+    if old is None:
+        if len(sys.argv) < 3:
+            print 'Usage %s old_tarball new_tarball [diff_tarball]' % \
+                sys.argv[0]
+            sys.exit(0)
+        old = sys.argv[1]
+        new = sys.argv[2]
+        if len(sys.argv) > 3:
+            result = sys.argv[3]
+
+    old_tarball = base.TarFile.open(old)
+    new_tarball = base.TarFile.open(new)
+    old_name = os.path.split(old)[-1]
+    new_name = os.path.split(new)[-1]
+    root_old_name = '.'.join(old_name.split('.')[:-1])
+    root_new_name = '.'.join(new_name.split('.')[:-1])
+
+    if result is None:
+        result = '%s-to-%s.tgz' % (root_old_name, root_new_name)
+
+    
+    old_files = {}
+    for f in old_tarball.getmembers():
+        old_files[f.name] = f
+
+    tmp = mkdtemp()
+
+    _log('Selecting files')
+    for f in new_tarball.getmembers():
+        if f.name in old_files:
+            if f.isfile():
+                # if its a file checking the diff
+                old_file = old_files[f.name]
+                old_content = old_tarball.extractfile(old_file)
+                old_content = old_content.read()
+                new_content = new_tarball.extractfile(f).read()
+                if old_content == new_content:
+                    continue
+        new_tarball.extract(f, tmp)
+
+    # now writing the diff tarball
+    _log('Writing archive')
+    diff_tarball = base.TarFile.open(result, 'w:gz')
+    try:
+        for root, dirs, filenames in os.walk(tmp):
+            for filename in filenames:
+                path = join(root, filename)
+                arcname = path.replace(tmp, '')
+                diff_tarball.add(path, arcname, False)
+    finally:
+        diff_tarball.close()
+    _log('Diff done.')
+
 def archive_contents(archive, location, previous=None):
     """generates the tarball"""
     tar = base.TarFile.open(join(location, '..', archive), 'w:gz')
     dirname, name = os.path.split(location)
-    if previous is None:
-        # full archive, recurse on directory
+    try:
         for root, dirs, filenames in os.walk(location):
             for filename in filenames:
                 path = join(root, filename)
                 arcname = path.replace(location, name)
                 tar.add(path, arcname, False)
-    else:
-        # snapshot archive, we put only files
-        # that are not in the snapshot
-        current_snapshot = snapshot(location)
-        previous_snapshot = base.svn_cat(previous + '/.snapshot')
-        previous_snapshot = previous_snapshot.split('\n')
-
-        for file_ in current_snapshot:
-            if file_ in previous_snapshot:
-                continue
-            file_ = os.path.join(location, file_)
-            if os.path.isdir(file_ ):
-                for root, dirs, filenames in os.walk(file_):
-                    for filename in filenames:
-                        path = join(root, filename)
-                        arcname = path.replace(location, name)
-                        tar.add(path, arcname, False)
-            else:
-                arcname = file_.replace(location, name)
-                tar.add(file_, arcname, False)
-
-        # then all the files in root
-        for element in os.listdir(location):
-            path = join(location, element)
-            if os.path.isdir(path):
-                continue
-            arcname = path.replace(location, name)
-            tar.add(path, arcname, False)
-
-    tar.close()
+    finally:
+        tar.close()
 
 def set_option(filename, section, option, value):
     """Setting option."""
@@ -149,26 +178,15 @@ def get_option(filename, section, option):
     config.read([filename])
     return config.get(section, option)
 
-def snapshot(buildout):
-    """Will make a snapshot over downloads and eggs folders."""
-    snap = []
-    for folder in ('downloads', 'eggs', os.path.join('downloads', 'dist')):
-        cfolder = os.path.join(buildout, folder)
-        if not os.path.exists(cfolder):
-            continue
-        # just top folders
-        for dir_ in os.listdir(cfolder):
-            if dir_ in ('dist', '.svn', '.', '..'):
-                continue
-            snap.append(os.path.join(folder, dir_))
-    return snap
-
 def _python():
     """returns python bin"""
-    python = join('bin', 'python')
-    if not os.path.exists(os.path.realpath(python)):
-        return join('bin', 'python2.4')
-    return python
+    possible_locations = (join('bin', 'python'),
+                          join('bin', 'python2.4'),
+                          join('Scripts', 'python'))
+    for location in possible_locations:
+        if os.path.exists(location):
+            return location
+    return 'python'
 
 def deploy_release(path=None, target=None, archiving=None):
     """deploy a release in-place"""
@@ -237,19 +255,6 @@ def deploy_release(path=None, target=None, archiving=None):
         raise base.ReleaseError('Something went wrong in deployement')
     print '%s ok.' % binary
 
-    # snapshot process: lists all files that where created in
-    # downloads and eggs to commit them in .snapshot
-    snapshot_file = join(root, '.snapshot')
-    f = open(snapshot_file, 'w')
-    try:
-        f.write('\n'.join(snapshot('.')))
-    finally:
-        f.close()
-
-    # saving snapshot
-    base.svn_add('.snapshot')
-    base.svn_commit('adding snapshot')
-
     # archiving process: changes buildout.cfg and create a tarball
     if filename != 'buildout.cfg':
         set_option(buildout, 'buildout', 'extends', filename)
